/*= ====================================================================================
 *  Alarm - vysialci cast
 *==================================================================================================
*/

#define NRF9E5 1



#include "reg9e5.h"
#include "stdio.h"
#define HFREQ      0                // 0=433MHz, 1=868/915MHz
#define CHANNEL  115                // Channel number: f(MHz) = (422.4+CHANNEL/10)*(1+HFREQ)  433.9MHz (pro 115ku)
/*
433.2 - 108
433.7 - 113
434.2 - 118
434.7 - 123
*/

#define TIMEOUT    20               // 20ms time-out on LED
#define ON          1
#define OFF         0

#define VOLTSPY	2182					//1.95V pro 12 bit res a samo mereni


static volatile unsigned char t0lrel, t0hrel;

sbit LEDKA = P0 ^ 6;
sbit TST = P0 ^ 7;
unsigned char tmp,i, Port, Napajeni, Citadlo, Service, Tick, Prijem, POW;


void Delay100us(volatile unsigned char n)				// zpozdeni
{
    unsigned char i;
    while(n--)
        for(i=0;i<35;i++)
            ;
}


unsigned char SpiReadWrite(unsigned char b)			// SPI komunikace
{
    EXIF &= ~0x20;                  // Clear SPI interrupt
    SPI_DATA = b;                   // Move byte to send to SPI data register
    while((EXIF & 0x20) == 0x00)    // Wait until SPI hs finished transmitting
        ;
    return SPI_DATA;
}


void ReceivePacket(unsigned int KolikCekat)						//prijme 4 pakety z TXu a je tam timeout
{
   
	TXEN = 0; 
   TRX_CE = 1;
   while((DR == 0) & (Tick < KolikCekat));
   if (Tick < KolikCekat)			//nebyl to timeout
   {
   	RACSN = 0;
	   SpiReadWrite(RRP);
		Port = SpiReadWrite(0);   			
		Napajeni = SpiReadWrite(0);   			
 		Citadlo = SpiReadWrite(0);
	 	Service = SpiReadWrite(0);
		RACSN = 1;
		Prijem = 1;		//neco prislo
	}
	else Prijem = 0;

   TRX_CE = 0;
}

void TransmitPacket(unsigned char Co)				// posle, co ma
{
	TXEN = 1;

    RACSN = 0;
    if (Co == 1)          //test sekvence
    {
    	SpiReadWrite(WTP);
    	SpiReadWrite(0xAA);			
    	SpiReadWrite(00);		
    	SpiReadWrite(0xAA);		
    	SpiReadWrite(00);	
    }
    else
    {
    	SpiReadWrite(WTP);
    	SpiReadWrite(Port);			
    	SpiReadWrite(Napajeni);		
    	SpiReadWrite(Citadlo);		
    	SpiReadWrite(Service);	
   
    }
    
    RACSN = 1;
    TRX_CE = 1;
    Delay100us(1);
    TRX_CE = 0;
    while(DR == 0);				//ceka, az se to posle
}


void InitTimer(void)
{
    TR0 = 0;
    TMOD &= ~0x03;
    TMOD |= 0x01;                   // mode 1
    CKCON |= 0x08;                  // T0M = 1 (/4 timer clock)
    t0lrel = 0x17;                  // 1KHz tick...
    t0hrel = 0xFC;                  // ... = 65536-4e6/(4*1e3) = FC17h
    TF0 = 0;                        // Clear any pending Timer0 interrupts
    ET0 = 1;                        // Enable Timer0 interrupt
}

void Timer0ISR (void) interrupt 1
{
    TF0 = 0;                        // Clear Timer0 interrupt
    TH0 = t0hrel;                   // Reload Timer0 high byte
    TL0 = t0lrel;                   // Reload Timer0 low byte
    Tick++;
}


unsigned int Mer_1_kanal(unsigned char Kanal)       //meri jeden kanal ADC
{
unsigned char x, tlo, thi;
unsigned int vysledek = 0;

		for (x = 0; x <8; x++)                //delame 8 samplu
		{
        RACSN = 0;			        // Start ADC conversion na danem kanale
        SpiReadWrite(SAV  | Kanal);
        RACSN = 1;

        while(EOC == 0);			        // Wait until ADC conversion completes

        RACSN = 0;			        // Read Out ADC Value
        SpiReadWrite(RAD);
        tlo = SpiReadWrite(NOP);  //lo byte
        thi = SpiReadWrite(NOP);
        vysledek = vysledek + ((thi << 8) + tlo);
        RACSN = 1;
		}

	return(vysledek/8);
}


void Zkouska_dosahu (void)
{
    t0lrel = 0x17;                  // 1KHz tick...
    t0hrel = 0xFC;                  // ... = 65536-4e6/(4*1e3) = FC17h

	LEDKA = 1;				//zhasni LED
	Tick = 0;
	TR0 = 1;
	while(1)
	{                   
	
		ReceivePacket(100);
		if (Prijem == 1)
		{
			LEDKA = 0;					//rozni LED
			TransmitPacket(1);
		}
		else
		{
			LEDKA = 1;
		}
		Tick = 0;
	}
	


}

void Kalibruj_tick (void)						// pro 4kHz musi vychazet 1000 pro 8MHz xtal, 500 pro 4MHz
{
unsigned int per, h, per2;
unsigned int vysl;

	T2CON = 0x0D;			// nastaveni capture
		TR2 = 0;
		TH2 = 0;
		TL2 = 0;
		TR2 = 1;
		T2CON = (T2CON & 0xBF); //smazne priznak	

		while ((T2CON & 0x40) == 0);
		per = (RCAP2H << 8) + RCAP2L;
		T2CON = (T2CON & 0xBF); //smazne priznak	
		while ((T2CON & 0x40) == 0);
		per2 = (RCAP2H << 8) + RCAP2L;
		vysl = (per2 - per);

		if (vysl > 600) TICK_DV = 0x02;			//kalibrace
		if ((vysl > 375) & (vysl < 450)) TICK_DV = 0x04;			//kalibrace			
		if (vysl <= 375) TICK_DV = 0x05;			//kalibrace			

/*
		if (vysl > 1200) TICK_DV = 0x02;			//kalibrace
		if ((vysl > 750) & (vysl < 900)) TICK_DV = 0x04;			//kalibrace			
		if (vysl <= 750) TICK_DV = 0x05;			//kalibrace			
*/		

/* 
- musi se udelat kalibrace v v TICK_DV
- Ttick = (1+hodnota)/fCLK
900-1100 je ok
1200 az 1400 dat hodnotu 2 (tzn +1 = 3)
800 - 900 - dat 4ku (tj 5)
pod 750 tak dat 5ku 

*/



}


void PowerSet(void)
{

//	POW = Mer_1_kanal(0);		//nemuzu mit externi trimr protoze to zere
	POW = 3;

    RACSN = 0;
    SpiReadWrite(WRC);              // Write RF config address 0
    SpiReadWrite(CHANNEL & 0xff);   // CHANNEL bit7..0
    // Change power defined by POWER above, to 433 or 868/915MHz defined by HFREQ and
    // bit8 of CHANNEL:
    SpiReadWrite(tmp | (POW<<2) | (HFREQ << 1) | ((CHANNEL >> 8) & 0x01));
    RACSN = 1;


}

void InitADC(unsigned char Akce)
{
    // Configure and turn on ADC
    RACSN = 0;
    SpiReadWrite(WAC);              // Write to ADC config
	 if (Akce == 1) SpiReadWrite(0x35);             // Select AIN3, PWR_UP = 1, VFSEL=0;
	 else SpiReadWrite(0x31) ;						//vypni ADC
    SpiReadWrite(0x09);             // Set RES_CTRL = 3, Right justified, 8 bitu
    RACSN = 1;
}

void Init(void)
{


    P0_DIR = 0xBF;                  // vse jako vstupy, 6ka LED out
//    P0 = 0x89;                      // P0.3 = 1 and P0.6=0 for the rec/tran selection
  
    SPICLK = 1;
    SPI_CTRL = 0x02;                // Connect internal SPI controller to Radio
    
    TH1 = 243;          // 9600@8MHz (when T1M=1 and SMOD=1)
    CKCON |= 0x30;      // T1M=1 (/4 timer clock) a TIM2 to same
    PCON = 0x80;        // SMOD=1 (double baud rate)
    SCON = 0x52;        // Serial mode1, enable receiver
    TMOD = 0x20;        // Timer1 8bit auto reload 
    TR1 = 1;            // Start timer1
    P0_ALT |= 0x00;     // nic
//    P0_ALT |= 0x04;     // TXD nastav - pro seriovku
	P0_DRV = 0x40;			// horni pin pro led
//    P0_DIR |= 0x02;     // P0.1 (RxD) is input


    RACSN = 0;
    SpiReadWrite(RRC | 0x09);
    tmp = SpiReadWrite(0) | 0x04;						//1110 0111 je default, ma byt 1100 1111
    
    RACSN = 1;

    RACSN = 0;
    SpiReadWrite(WRC | 0x09);			
    SpiReadWrite(0xC7);				// 4MHz
    RACSN = 1;

    // Configure Radio:
    RACSN = 0;
    SpiReadWrite(WRC | 0x03);       // Write to RF config address 3 (RX payload)
    SpiReadWrite(0x04);             // 3 byte RX payload
    SpiReadWrite(0x04);             // 3 byte TX payload
    RACSN = 1;

    RACSN = 0;
    SpiReadWrite(RRC | 0x01);       // Read RF config address 1
    tmp = SpiReadWrite(0) & 0xf0;   // Clear the power and frequency setting bits
    RACSN = 1;


    InitTimer();
    EA = 1;
}

void main(void)
{


   Init();
	InitADC(0);
	PowerSet();				//nastavi vykon
	Kalibruj_tick();
	if (~TST) Zkouska_dosahu();				//test dosahu

		while ((REGX_CTRL & 0x10) != 0);			
		REGX_MSB = 0xDD;						// na vsech pinech citlivost na hranu
		REGX_LSB = 0xDD;						// a filtr 2 LF clk
		REGX_CTRL = 0x0C;						// pisem GPIO probuzeni WWCON0

		while ((REGX_CTRL & 0x10) != 0);			
		REGX_MSB = 0x00;						// na vsech spodnich pinech citlivost na hranu
		REGX_LSB = 0xDD;						// a filtr 2 LF clk
		REGX_CTRL = 0x0D;						// pisem GPIO probuzeni WWCON1
		

	while(1)
	{

		while ((REGX_CTRL & 0x10) != 0);			//cekej, az se ten reg uvolni
		REGX_MSB = 0xEA;							//cas idle cca 60 sec
		REGX_LSB = 0x60;
		REGX_CTRL = 0x0A;						// zapis to do RTC registru
                                                           //timer latch

		LEDKA = 1;
		while ((REGX_CTRL & 0x10) != 0);			//cekej, az se ten reg uvolni - podle datasheetu
		CK_CTRL = 0x04;						// power down?

// *********** tady se probudil
		LEDKA = 0;                               
		while ((REGX_CTRL & 0x10) != 0);			//cekej, az se ten reg uvolni
		REGX_CTRL = 0x0B;						// zakaz RTC WRTCDIS


		InitADC(1);						//zapne adc
		Port = (P0 & 0x3F);						// vezme hodnotu portu maskuje vrch 
		Napajeni = Mer_1_kanal (8);
		Citadlo++;
		InitADC(0);						//vypne ADC

//	    TXEN = 1;
		for (i = 0; i < 4; i++)		//posle data 4x
		{
	    	TransmitPacket(0);
		   Delay100us(1);
		}
	}
}

