/*= ====================================================================================
 *  Alarm prijimaci cast
 *==================================================================================================
*/

#define NRF9E5 1


#include "reg9e5.h"
#include "stdio.h"
#define HFREQ      0                // 0=433MHz, 1=868/915MHz
#define CHANNEL  115                // Channel number: f(MHz) = (422.4+CHANNEL/10)*(1+HFREQ)  433.9MHz (pro 115ku)

/*
Nektere kmitocty
433.2 - 108
433.7 - 113
434.2 - 118
434.7 - 123
*/


#define TIMEOUT    20               // 20ms time-out on LED
#define ON          1
#define OFF         0

#define VOLTSPY	2182					//1.95V threshold pro 12 bit res a samo mereni (3x delic uvnitr)


static volatile unsigned char t0lrel, t0hrel;		//promenne casovace

sbit DAT = P0 ^ 4;			//datovy pin SIPO
sbit CLK = P0 ^ 1;			//clk pin SIPO
sbit RES = P0 ^ 0;			//reset SIPO registru
sbit TST = P0 ^ 6;			//test pin
sbit REZ = P0 ^ 5;			//volba rezimu
sbit OPTO = P0 ^ 3;			//vystupni optoclen

unsigned char tmp,i, Port, Napajeni, Citadlo, Service, Prijem, POW;
unsigned long int Tick;
unsigned char AlarmSpoj, AlarmNap, AlarmSmyc;



void Delay100us(volatile unsigned char n)        //ceka cca 100us
{
    unsigned char i;
    while(n--)
        for(i=0;i<35;i++)
            ;
}


unsigned char SpiReadWrite(unsigned char b)			//zapis do SPI registru - dle prikladu Nordic
{
    EXIF &= ~0x20;                  // Clear SPI interrupt
    SPI_DATA = b;                   // Move byte to send to SPI data register
    while((EXIF & 0x20) == 0x00)    // Wait until SPI hs finished transmitting
        ;
    return SPI_DATA;
}

void Led_driver (unsigned char CoSviti)		//ridi 74164 displej - SIPO registr
{
unsigned char xy;

	CLK = 0;			//pocatecni stav
	RES = 0;			//udela reset - pro jistotu
	_nop_();			//chvili podrz
	RES = 1;			//reset hotov
	_nop_();
	
	for (xy = 0; xy <8; xy++)							//posli 8 bitu, jak se ma svitit
	{
		if ((CoSviti & 0x01) == 0x01) DAT = 1;
		else DAT = 0;
		_nop_();
		CLK = 1;				//hodinovy puls
		_nop_();
		CLK = 0;
		CoSviti = (CoSviti >> 1);	
	}

/*
Jak jsou ty LED zapojeny na plosnaku:

cislo
7       stav 6ti vstupu
6
5
4
3
2
      0     1

*/
	
}

void ReceivePacket(unsigned long int KolikCekat)				//prijme 4 pakety z TXu, zaroven tam bezi timeout - 
{
   
	TXEN = 0; 			//bude prijem
   TRX_CE = 1;			//zapni TRX
   while((DR == 0) & (Tick < KolikCekat));				//cekej az neco prijde nebo az vyprsi cas
   if (Tick < KolikCekat)			//nebyl to timeout, neco prislo
   {
   	RACSN = 0;
	   SpiReadWrite(RRP);
		Port = SpiReadWrite(0);   			//prvni bajt
		Napajeni = SpiReadWrite(0);   	//druhy....
 		Citadlo = SpiReadWrite(0);
	 	Service = SpiReadWrite(0);
		RACSN = 1;
		Prijem = 1;		//neco prislo
	}
	else Prijem = 0;

   TRX_CE = 0;       //vypni TRX

}

void TransmitPacket(void)				// posle, co ma - AA a 00 jako testovaci bajty pro test dosahu
{
	
	TXEN = 1;
    RACSN = 0;
    SpiReadWrite(WTP);
    SpiReadWrite(0xAA);			
    SpiReadWrite(00);		
    SpiReadWrite(0xAA);		
    SpiReadWrite(00);		

    RACSN = 1;
    TRX_CE = 1;
    Delay100us(1);
    TRX_CE = 0;
    while(DR == 0);				//ceka, az se to posle
}


void InitTimer(void)               //init casovace
{
    TR0 = 0;
    TMOD &= ~0x03;
    TMOD |= 0x01;                   // mode 1
    CKCON |= 0x08;                  // T0M = 1 (/4 timer clock)
    t0lrel = 0x2F;                  // 1KHz tick...
    t0hrel = 0xF8;                  // ... = 65536-4e6/(4*1e3) = F82Fh
    TF0 = 0;                        // Clear any pending Timer0 interrupts
    ET0 = 1;                        // Enable Timer0 interrupt
}

void Timer0ISR (void) interrupt 1
{
    TF0 = 0;                        // Clear Timer0 interrupt
    TH0 = t0hrel;                   // Reload Timer0 high byte
    TL0 = t0lrel;                   // Reload Timer0 low byte
    Tick++;
}


unsigned int Mer_1_kanal(unsigned char Kanal)		//zmeri hodnotu na danem kanale
{
unsigned char x, tlo, thi;
unsigned int vysledek = 0;

		for (x = 0; x <8; x++)                //delame 8 samplu
		{
        RACSN = 0;			        // Start ADC conversion na danem kanale
        SpiReadWrite(SAV  | Kanal);
        RACSN = 1;

        while(EOC == 0);			        // Wait until ADC conversion completes

        RACSN = 0;			        // Read Out ADC Value
        SpiReadWrite(RAD);
        tlo = SpiReadWrite(NOP);  //lo byte
        thi = SpiReadWrite(NOP);
        vysledek = vysledek + ((thi << 8) + tlo);
        RACSN = 1;
		}

	return(vysledek/8);					//prumerujem
}



void Zkouska_dosahu (void)						//zkouska dosahu
{
    t0lrel = 0x17;                  // 1KHz tick...
    t0hrel = 0xFC;                  // ... = 65536-4e6/(4*1e3) = FC17h

	Led_driver(0);				//zhasne danou ledku
	TR0 = 1;						// zapne timer

	while(1)
	{
		TransmitPacket();			//posle neco
		Tick = 0;
		ReceivePacket(100);			//prijima a zaroven ceka
		if (Prijem == 0) Led_driver(0);
		else Led_driver (0x40);			//rozsvit LED ze neco prislo
		Delay100us(1);
	}

}


void Kalibruj_tick (void)						
{
unsigned int per, h, per2;
unsigned int vysl;
// pro 4kHz musi vychazet 1000 pro 8MHz xtal, 500 pro 4MHz
// kalibruje se interni RC oscilator
/* 
- musi se udelat kalibrace v v TICK_DV
- Ttick = (1+hodnota)/fCLK
900-1100 je ok
1200 az 1400 dat hodnotu 2 (tzn +1 = 3)
800 - 900 - dat 4ku (tj 5)
pod 750 tak dat 5ku 

*/



	T2CON = 0x0D;			// nastaveni capture
		TR2 = 0;
		TH2 = 0;
		TL2 = 0;
		TR2 = 1;
		T2CON = (T2CON & 0xBF); //smazne priznak	

		while ((T2CON & 0x40) == 0);
		per = (RCAP2H << 8) + RCAP2L;
		T2CON = (T2CON & 0xBF); //smazne priznak	
		while ((T2CON & 0x40) == 0);
		per2 = (RCAP2H << 8) + RCAP2L;
		vysl = (per2 - per);

/*
		if (vysl > 600) TICK_DV = 0x02;			//kalibrace pro 4MHz
		if ((vysl > 375) & (vysl < 450)) TICK_DV = 0x04;			//kalibrace			
		if (vysl <= 375) TICK_DV = 0x05;			//kalibrace			
*/

		if (vysl > 1200) TICK_DV = 0x02;			//kalibrace pro 8MHz xtal
		if ((vysl > 750) & (vysl < 900)) TICK_DV = 0x04;			//kalibrace			
		if (vysl <= 750) TICK_DV = 0x05;			//kalibrace			
		

}
             
void PowerSet(void)
{


	POW = 3;				//nastaveni vykonu, 0 je min, 3 je max
    RACSN = 0;
    SpiReadWrite(WRC);              // Write RF config address 0
    SpiReadWrite(CHANNEL & 0xff);   // CHANNEL bit7..0
    // Change power defined by POWER above, to 433 or 868/915MHz defined by HFREQ and
    // bit8 of CHANNEL:
    SpiReadWrite(tmp | (POW<<2) | (HFREQ << 1) | ((CHANNEL >> 8) & 0x01));
    RACSN = 1;


}

void InitADC(unsigned char Akce)
{
    // Configure and turn on ADC, je-li akce = 0, vypne ADC
    
    RACSN = 0;
    SpiReadWrite(WAC);              // Write to ADC config
	 if (Akce == 1) SpiReadWrite(0x35);             // Select AIN3, PWR_UP = 1, VFSEL=0;
	 else SpiReadWrite(0x31) ;						//vypni ADC
    SpiReadWrite(0x09);             // Set RES_CTRL = 3, Right justified, 8 bitu
    RACSN = 1;
}

void Init(void)			//zakladni nastaveni cipu
{


    P0_DIR = 0x60;                  // 6,5 jsou in, zbytek out
 
    SPICLK = 1;
    SPI_CTRL = 0x02;                // Connect internal SPI controller to Radio
    
    TH1 = 243;          // 9600@8MHz (when T1M=1 and SMOD=1)
    CKCON |= 0x30;      // T1M=1 (/4 timer clock) a TIM2 to same
    PCON = 0x80;        // SMOD=1 (double baud rate)
    SCON = 0x52;        // Serial mode1, enable 
    TMOD = 0x20;        // Timer1 8bit auto reload 
    TR1 = 1;            // Start timer1
    P0_ALT |= 0x04;     // TXD nastav - pro seriovku, alternativni funkce
	 P0_DRV = 0x08;		//P0.4 dava proud pro optoclen


    // Switch to 16MHz clock:
    RACSN = 0;
    SpiReadWrite(RRC | 0x09);
    tmp = SpiReadWrite(0) | 0x04;						//1110 0111 je default, ma byt 1100 1111
    
    RACSN = 1;

    RACSN = 0;
    SpiReadWrite(WRC | 0x09);			// 9. bajt
    SpiReadWrite(0xCF);				// 8MHz
//    SpiReadWrite(0xC7);				// 4MHz
    RACSN = 1;

    // Configure Radio:
    RACSN = 0;
    SpiReadWrite(WRC | 0x03);       // Write to RF config address 3 (RX payload)
    SpiReadWrite(0x04);             // 4 byte RX payload
    SpiReadWrite(0x04);             // 4 byte TX payload
    RACSN = 1;

    RACSN = 0;
    SpiReadWrite(RRC | 0x01);       // Read RF config address 1
    tmp = SpiReadWrite(0) & 0xf0;   // Clear the power and frequency setting bits
    RACSN = 1;

    InitTimer();
    EA = 1;
}

void main(void)
{

   Init();
	InitADC(1);				//zapne prevodnik
	PowerSet();				//nastavi vykon
	
	Kalibruj_tick();		//kalibruje RC oscilator


	if (~TST) Zkouska_dosahu();				//test dosahu
	Port = 0x40;			//ztrata spojeni - default
	Led_driver (Port);		// da stav ledek 

	OPTO = 0;
	TR0 = 1;						// zapne timer
	while(1)
	{
		TXEN = 0;
		Tick = 0;
      ReceivePacket(70000);			//70000 - cekej 70sec cca
      
		if (Prijem == 1)
		{
			if ((Port & 0x3F) == 0x38)
			{
				AlarmSmyc = 0;				//alarm smycky
			}
			else AlarmSmyc = 1;
	
			AlarmSpoj = 0;			//vse je dobre spojeno
			
			i = (Port & 0x07);		//pamatuje si cba
			Port = (~Port);				//neguje
			Port = (Port & 0x38);	//nuly na lsb
			Port = (Port | i);		//pricte cba
			if (Napajeni < 140)		// 140 je 2.1V (1.22V ref, x3, 8 bitu)   kontrola napajeni!
			{
				Port = (Port | 0x80);       // male napeti
				AlarmNap = 1;
			}
			else
			{
				Port = (Port & 0xBF);
				AlarmNap = 0;
			}
		}
		else
		{
	 		Port = 0x40;			//ztrata spojeni!!!
			AlarmSpoj = 1;			//chyba ztrata spojeni
		}	
      Led_driver (Port);		// da stav ledek 
      
      if (((AlarmSpoj == 1) | (AlarmNap == 1)) | (AlarmSmyc == 1)) OPTO = 1;
      else if (~REZ) OPTO = 0;		//zhasne pouze je-li REZ v nule
      
	}

}

