/*--------------------------------------------------------------------------
 * reg9e5.h
 *
 * C51 header file for the Nordic VLSI nRF9E5 433-968MHz RF transceiver
 * with embedded 8051 compatible microcontroller.
 *
 * $Revision: 2 $
 *
 *------------------------------------------------------------------------*/

/*  BYTE Registers  */

sfr at 0x80 P0		;
sfr at 0x81 SP          ;
sfr at 0x82 DPL         ;
sfr at 0x83 DPH         ;
sfr at 0x84 DPL1        ;
sfr at 0x85 DPH1        ;
sfr at 0x86 DPS         ;
sfr at 0x87 PCON        ;
sfr at 0x88 TCON        ;
sfr at 0x89 TMOD        ;
sfr at 0x8A TL0         ;
sfr at 0x8B TL1         ;
sfr at 0x8C TH0         ;
sfr at 0x8D TH1         ;
sfr at 0x8E CKCON       ;
sfr at 0x90 P1          ;
sfr at 0x91 EXIF        ;
sfr at 0x92 MPAGE       ;
sfr at 0x98 SCON        ;
sfr at 0x99 SBUF        ;
sfr at 0xA8 IE          ;
sfr at 0xB8 IP          ;
sfr at 0xC8 T2CON       ;
sfr at 0xCA RCAP2L      ;
sfr at 0xCB RCAP2H      ;
sfr at 0xCC TL2         ;
sfr at 0xCD TH2         ;
sfr at 0xD0 PSW         ;
sfr at 0xD8 EICON       ;
sfr at 0xE0 ACC         ;
sfr at 0xF0 B           ;
sfr at 0xE8 EIE         ;
sfr at 0xF8 EIP         ;

sfr at 0x93 P0_DRV;
sfr at 0x94 P0_DIR      ;
sfr at 0x95 P0_ALT      ;
sfr at 0x96 P1_DIR      ;
sfr at 0x97 P1_ALT      ;
sfr at 0xA0 P2          ;
sfr at 0xA9 PWMCON      ;
sfr at 0xAA PWMDUTY     ;
sfr at 0xAB REGX_MSB    ;
sfr at 0xAC REGX_LSB    ;
sfr at 0xAD REGX_CTRL   ;
sfr at 0xB1 RSTREAS     ;
sfr at 0xB2 SPI_DATA    ;
sfr at 0xB3 SPI_CTRL    ;
sfr at 0xB4 SPICLK      ;
sfr at 0xB5 TICK_DV     ;
sfr at 0xB6 CK_CTRL     ;
sfr at 0xBF CKLFCON     ;

/*  BIT Registers  */

/*  PSW */
sbit CY         = PSW^7;
sbit AC         = PSW^6;
sbit F0         = PSW^5;
sbit RS1        = PSW^4;
sbit RS0        = PSW^3;
sbit OV         = PSW^2;
sbit F1         = PSW^1;
sbit P          = PSW^0;

/*  TCON  */
sbit TF1        = TCON^7;
sbit TR1        = TCON^6;
sbit TF0        = TCON^5;
sbit TR0        = TCON^4;
sbit IE1        = TCON^3;
sbit IT1        = TCON^2;
sbit IE0        = TCON^1;
sbit IT0        = TCON^0;

/*  IE  */
sbit EA         = IE^7;
sbit ET2        = IE^5;
sbit ES         = IE^4;
sbit ET1        = IE^3;
sbit EX1        = IE^2;
sbit ET0        = IE^1;
sbit EX0        = IE^0;

/*  IP  */
sbit PT2        = IP^5;
sbit PS         = IP^4;
sbit PT1        = IP^3;
sbit PX1        = IP^2;
sbit PT0        = IP^1;
sbit PX0        = IP^0;


/*  P0  */
sbit T1         = P0^6;
sbit T0         = P0^5;
sbit INT1       = P0^4;
sbit INT0       = P0^3;

/*  P1  */
sbit T2         = P1^0;
sbit EECSN      = P1^3;

/*  SCON  */
sbit SM0        = SCON^7;
sbit SM1        = SCON^6;
sbit SM2        = SCON^5;
sbit REN        = SCON^4;
sbit TB8        = SCON^3;
sbit RB8        = SCON^2;
sbit TI         = SCON^1;
sbit RI         = SCON^0;

/*  T2CON  */
sbit TF2        = T2CON^7;
sbit EXF2       = T2CON^6;
sbit RCLK       = T2CON^5;
sbit TCLK       = T2CON^4;
sbit EXEN2      = T2CON^3;
sbit TR2        = T2CON^2;
sbit C_T2       = T2CON^1;
sbit CP_RL2     = T2CON^0;

/*  EICON  */
sbit SMOD1      = EICON^7;
sbit WDTI       = EICON^3;

/*  EIE  */
sbit EWDI       = EIE^4;
sbit EX5        = EIE^3;
sbit EX4        = EIE^2;
sbit EX3        = EIE^1;
sbit EX2        = EIE^0;

/*  EIP  */
sbit PWDI       = EIP^4;
sbit PX5        = EIP^3;
sbit PX4        = EIP^2;
sbit PX3        = EIP^1;
sbit PX2        = EIP^0;

/* RADIO Port (P2) */

sbit SPI_SCK    = P2^0;
sbit SPI_MOSI   = P2^1;
sbit RACSN      = P2^3;
sbit TXEN       = P2^4;
sbit TRX_CE     = P2^5;

sbit SPI_MISO   = P2^2;
sbit EOC        = P2^4;
sbit DR         = P2^5;
sbit CD         = P2^6;
sbit AM         = P2^7;

sbit P00        = P0^0;
sbit P01        = P0^1;
sbit P02        = P0^2;
sbit P03        = P0^3;
sbit P04        = P0^4;
sbit P05        = P0^5;
sbit P06        = P0^6;
sbit P07        = P0^7;

// SPI Commands

#define WRC     0x00 //W_RF_CONFIG
#define RRC     0x10 //R_RF_CONFIG
#define WTP     0x20 //W_TX_PAYLOAD
#define RTP     0x21 //R_TX_PAYLOAD
#define WTA     0x22 //W_TX_ADDRESS
#define RTA     0x23 //R_TX_ADDRESS
#define RRP     0x24 //R_RX_PAYLOAD
#define RAD     0x40 //R_ADC_DATA 
#define WAC     0x44 //W_ADC_CONFIG
#define RAC     0x46 //R_ADC_CONFIG
#define WTU     0x50 //W_TEST_UNLOCK (use with data A5)
#define WTR     0x52 //W_TEST_REGISTER
#define RTR     0x53 //R_TEST_REGISTER
#define CC      0x80 //CHANNEL_CONFIG
#define SAV     0xC0 //START_ADC_CONV
#define NOP     0xFF //NO_OP



